@@ -2,10 +2,12 @@ from feast import FeatureStore, RepoConfig
 
 from amora.feature_store.config import settings
 from amora.feature_store.logging import patch_tqdm
+from amora.feature_store.online_store import patch_online_store
 from amora.feature_store.usage_tracking import patch_usage
 
 patch_usage()
 patch_tqdm()
+patch_online_store()
 
 repo_config = RepoConfig(
     registry=settings.REGISTRY,
