@@ -1,23 +1,18 @@
+import itertools
 import json
 import os
+from typing import Callable, Iterable, Optional
+
 import pytest
-from typing import Iterable, Optional, Callable, Union
-from sqlalchemy import (
-    and_,
-    union_all,
-    Integer,
-    func,
-    literal,
-)
+from sqlalchemy import ARRAY, Integer, and_, func, literal, or_, union_all
 from sqlmodel.sql.expression import SelectOfScalar
+
 from amora.config import settings
-from amora.models import select, AmoraModel, Session, Column, Columns
+from amora.models import AmoraModel, ColumnElement, Session, select
+from amora.protocols import Compilable
+from amora.providers.bigquery import RunResult, estimated_query_cost_in_usd, run
 from amora.storage import local_engine
 from amora.tests.audit import AuditLog
-from amora.types import Compilable
-from amora.providers.bigquery import run, RunResult, estimated_query_cost_in_usd
-from collections import defaultdict
-
 
 Test = Callable[..., SelectOfScalar]
 
@@ -50,7 +45,8 @@ def _test(statement: Compilable, raise_on_fail: bool = True) -> bool:
 
     if run_result.rows.total_rows == 0:
         return True
-    elif raise_on_fail:
+
+    if raise_on_fail:  # pragma: nocover
         pytest.fail(
             f"{run_result.rows.total_rows} rows failed the test assertion."
             f"\n==========="
@@ -59,12 +55,12 @@ def _test(statement: Compilable, raise_on_fail: bool = True) -> bool:
             f"\n{run_result.query}",
             pytrace=False,
         )
-    else:
-        return False
+
+    return False
 
 
 def that(
-    column: Column,
+    column: ColumnElement,
     test: Test,
     raise_on_fail: bool = True,
     **test_kwargs,
@@ -86,7 +82,7 @@ def that(
     return _test(statement=test(column, **test_kwargs), raise_on_fail=raise_on_fail)
 
 
-def is_not_null(column: Column) -> Compilable:
+def is_not_null(column: ColumnElement) -> Compilable:
     """
     Asserts that the `column` does not contain `null` values
 
@@ -108,7 +104,7 @@ def is_not_null(column: Column) -> Compilable:
     return select(column).where(column == None)
 
 
-def is_unique(column: Column) -> Compilable:
+def is_unique(column: ColumnElement) -> Compilable:
     """
     Assert that the `column` values are unique
 
@@ -134,7 +130,7 @@ def is_unique(column: Column) -> Compilable:
     return select(column).group_by(column).having(func.count(column) > 1)
 
 
-def has_accepted_values(column: Column, values: Iterable) -> Compilable:
+def has_accepted_values(column: ColumnElement, values: Iterable) -> Compilable:
     """
     Assert that the values from the `column` should be one of the provided `values`
 
@@ -156,8 +152,8 @@ def has_accepted_values(column: Column, values: Iterable) -> Compilable:
 
 
 def relationship(
-    from_: Column,
-    to: Column,
+    from_: ColumnElement,
+    to: ColumnElement,
     from_condition=None,
     to_condition=None,
 ) -> bool:
@@ -240,7 +236,7 @@ def relationship(
     return _test(statement=exceptions)
 
 
-def is_numeric(column: Column) -> Compilable:
+def is_numeric(column: ColumnElement) -> Compilable:
     """
     Asserts that each not null value is a number
 
@@ -258,13 +254,13 @@ def is_numeric(column: Column) -> Compilable:
     Example:
 
     ```python
-    is_numeric(func.cast(Health.value, String).label('value_as_str'))
+    is_numeric(func.cast(Health.value, String).label("value_as_str"))
     ```
     """
     return select(column).where(func.REGEXP_CONTAINS(column, "[^0-9]"))
 
 
-def is_non_negative(column: Column) -> Compilable:
+def is_non_negative(column: ColumnElement) -> Compilable:
     """
     Asserts that every column value should be >= 0
 
@@ -285,7 +281,7 @@ def is_non_negative(column: Column) -> Compilable:
     return select(column).where(column < 0)
 
 
-def is_a_non_empty_string(column: Column) -> Compilable:
+def is_a_non_empty_string(column: ColumnElement) -> Compilable:
     """
     Asserts that the column isn't an empty string
 
@@ -338,7 +334,7 @@ def expression_is_true(expression, condition=None) -> bool:
 def equality(
     model_a: AmoraModel,
     model_b: AmoraModel,
-    compare_columns: Optional[Iterable[Column]] = None,
+    compare_columns: Optional[Iterable[ColumnElement]] = None,
 ) -> bool:
     """
     This schema test asserts the equality of two models. Optionally specify a subset of columns to compare.
@@ -347,7 +343,7 @@ def equality(
 
     raise NotImplementedError
 
-    def comparable_columns(model: AmoraModel) -> Iterable[Column]:
+    def comparable_columns(model: AmoraModel) -> Iterable[ColumnElement]:
         if not compare_columns:
             return model
         return [getattr(model, column_name) for column_name in compare_columns]
@@ -364,7 +360,7 @@ def equality(
     return _test(statement=diff_union)
 
 
-def has_at_least_one_not_null_value(column: Column) -> Compilable:
+def has_at_least_one_not_null_value(column: ColumnElement) -> Compilable:
     """
     Asserts if column has at least one value.
 
@@ -388,7 +384,7 @@ def has_at_least_one_not_null_value(column: Column) -> Compilable:
     return select(func.count(column, type_=Integer)).having(func.count(column) == 0)
 
 
-def are_unique_together(columns: Iterable[Column]) -> Compilable:
+def are_unique_together(columns: Iterable[ColumnElement]) -> Compilable:
     """
     This test confirms that the combination of columns is unique.
     For example, the combination of month and product is unique,
@@ -402,3 +398,35 @@ def are_unique_together(columns: Iterable[Column]) -> Compilable:
 
     """
     return select(columns).group_by(*columns).having(func.count(type_=Integer) > 1)
+
+
+def has_the_same_array_length(columns: Iterable[ColumnElement[ARRAY]]) -> Compilable:
+    """
+    Asserts that all array columns has the same length
+
+    Example SQL:
+
+    ```sql
+    SELECT
+        arr_col1,
+        arr_col2,
+        arr_col3
+    FROM
+        {{ model }}
+    WHERE
+        ARRAY_LENGTH(arr_col1) != ARRAY_LENGTH(arr_col2)
+        OR ARRAY_LENGTH(arr_col1) != ARRAY_LENGTH(arr_col3)
+        OR ARRAY_LENGTH(arr_col2) != ARRAY_LENGTH(arr_col3)
+    ```
+
+    Example:
+
+    ```python
+    assert that([Model.arr_col1, Model.arr_col2, Model.arr_col3], has_the_same_array_length)
+    ```
+    """
+    conditions = (
+        func.array_length(a) != func.array_length(b)
+        for (a, b) in itertools.combinations(columns, 2)
+    )
+    return select(columns).where(or_(*conditions))
